<!--Copyright 2019 Google LLC-->
<!-- -->
<!--Licensed under the Apache License, Version 2.0 (the "License");-->
<!--you may not use this file except in compliance with the License.-->
<!--You may obtain a copy of the License at-->
<!-- -->
<!--     https://www.apache.org/licenses/LICENSE-2.0-->
<!-- -->
<!--Unless required by applicable law or agreed to in writing, software-->
<!--distributed under the License is distributed on an "AS IS" BASIS,-->
<!--WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.-->
<!--See the License for the specific language governing permissions and-->
<!--limitations under the License.-->

<html>
<head>
    <title>unfurl</title>

    <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.min.js"></script>
    <link href="https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.min.css" rel="stylesheet" type="text/css" />
    <link rel="stylesheet" type="text/css" href="/static/styles.css">

    <link rel="apple-touch-icon" sizes="180x180" href="/static/apple-touch-icon.png">
    <link rel="icon" type="image/png" sizes="32x32" href="/static/favicon-32x32.png">
    <link rel="icon" type="image/png" sizes="16x16" href="/static/favicon-16x16.png">
    <link rel="manifest" href="/static/site.webmanifest">

</head>
<body>
    <header class="top-bar">
        <table class="top-bar-table">
            <tr>
                <td><a href="/"><span class="logo-text">unfurl&nbsp</span></a></td>
            </tr>
        </table>
    </header>
    <div class="main-body-wrapper">
    <div class="main-body">
        {% block content %}{% endblock %}
    </div>
    </div>
    </body>
</html>
